@@ -124,6 +124,8 @@ def evalglare(ctx, arg1, **kwargs):
 @click.option('-idx', default=0,
               help='column to build kde from')
 @click.option('-resample', callback=clk.split_float)
+@click.option('-scale', type=int)
+@click.option('-sigma', type=float)
 @clk.shared_decs(clk.command_decs(hs.__version__))
 def kde(ctx, arg1, **kwargs):
     """generate KDE for samples"""
@@ -134,9 +136,13 @@ def kde(ctx, arg1, **kwargs):
     else:
         try:
             aS = np.loadtxt(arg1)[:,kwargs['idx']]
-            ssize = hs.kde_cont(aS, kwargs['resample'])
+            ssize = hs.kde_cont(aS, kwargs['resample'], kwargs['sigma'])
+            if kwargs['scale'] is not None:
+                scale = kwargs['scale']/len(aS)
+            else:
+                scale = 1
             for i in ssize:
-                print(i[0], i[1])
+                print(i[0], i[1]*scale)
         except click.Abort:
             raise
         except Exception as ex:
@@ -144,6 +150,37 @@ def kde(ctx, arg1, **kwargs):
     return 'kde', kwargs, ctx
 
 
+def get_baseline(aS, i, maxN=10000):
+    if i < aS.shape[-1]:
+        a = aS[:,i]
+    else:
+        a = aS.flatten()
+    la = len(a)
+    if la > maxN:
+        scale = la/maxN
+        select = np.random.randint(la, size=maxN)
+        a = a[select]
+    else:
+        scale = 1
+        select = None
+    return a, scale, select
+
+
+def get_rdif(aS, a2S, i, select=None, relative=True):
+    if i < aS.shape[-1]:
+        a = np.stack((aS[:,i],a2S[:,i])).T
+    else:
+        a = np.stack((aS.flatten(),a2S.flatten())).T
+    la = len(a)
+    if select is not None:
+        a = a[select]
+    b = a
+    if relative:
+        return b[:,1]/b[:,0] - 1
+    else:
+        return b[:,1] - b[:,0]
+
+
 @main.command()
 @click.argument('arg1', callback=clk.is_file)
 @click.argument('arg2', callback=clk.is_file)
@@ -157,40 +194,11 @@ def kde(ctx, arg1, **kwargs):
 @click.option('--legacy/--no-legacy', default=False)
 @click.option('--relative/--absolute', default=True)
 @click.option('-rng', default=None, callback=clk.split_int)
+@click.option('-sigma', type=float)
 @clk.shared_decs(clk.command_decs(hs.__version__))
 def kde_error(ctx, arg1, arg2, outpref, **kwargs):
     """calculate error between 2 2D data files using gaussian KDE"""
     
-    def get_baseline(aS, i, maxN=10000):
-        if i < aS.shape[-1]:
-            a = aS[:,i]
-        else:
-            a = aS.flatten()
-        la = len(a)
-        if la > maxN:
-            scale = la/maxN
-            select = np.random.randint(la, size=maxN)
-            a = a[select]
-        else:
-            scale = 1
-            select = None
-        return a, scale, select
-
-
-    def get_rdif(aS, a2S, i, select=None, relative=True):
-        if i < aS.shape[-1]:
-            a = np.stack((aS[:,i],a2S[:,i])).T
-        else:
-            a = np.stack((aS.flatten(),a2S.flatten())).T
-        la = len(a)
-        if select is not None:
-            a = a[select]
-        b = a
-        if relative:
-            return b[:,1]/b[:,0] - 1
-        else:
-            return b[:,1] - b[:,0]
-    
     if kwargs['opts']:
         kwargs['opts'] = False
         clk.echo_args(arg1, arg2, outpref, **kwargs)
@@ -212,8 +220,9 @@ def kde_error(ctx, arg1, arg2, outpref, **kwargs):
                     baseline, scale, select = get_baseline(aS, i)
                 rdif = get_rdif(aS, a2S, i, select, kwargs['relative'])
                 adif = np.abs(rdif)
+                print(baseline.shape)
                 out = hs.error_cont(baseline, adif, rdif, scale=scale,
-                                  resample=kwargs['resample'])
+                                  resample=kwargs['resample'], ksigma=kwargs['sigma'])
                 ti = i + kwargs['sidx']
                 outf = f'{outpref}_{ti}.txt'
                 if kwargs['legacy']:
@@ -298,6 +307,89 @@ def corr(dataf, **kwargs):
     return 'corr', kwargs
 
 
+@main.command()
+@click.argument('arg1', callback=clk.is_file)
+@click.argument('arg2', callback=clk.is_file)
+@click.argument('outpref')
+@click.argument('coef', callback=clk.is_file)
+@click.option('-baselinei', type=int, default=0,
+              help='use i as baseline for KDE of all columns')
+@click.option('--relative/--absolute', default=True)
+@click.option('-rng', default=None, callback=clk.split_int)
+@clk.shared_decs(clk.command_decs(hs.__version__))
+def logist_error(ctx, arg1, arg2, outpref, coef, **kwargs):
+    """calculate error between 2 2D data files using softmax distributions from logistic regression"""
+    
+    if kwargs['opts']:
+        kwargs['opts'] = False
+        clk.echo_args(arg1, arg2, outpref, coef, **kwargs)
+    else:
+        try:
+            aS = np.loadtxt(arg1)
+            a2S = np.loadtxt(arg2)
+            coefs = np.loadtxt(coef)
+            labels = aS[:,0]
+            if kwargs['rng'] is None:
+                srng = range(aS.shape[-1])
+            else:
+                srng = np.array(kwargs['rng'])
+            for i in srng:
+                rdif = get_rdif(aS, a2S, i, relative=kwargs['relative'])
+                adif = np.abs(rdif)
+                print('\n',i)
+                out = hs.error_lgw(aS[:,kwargs['baselinei']], adif, rdif, coefs)
+                outf = f'{outpref}_{i}.txt'
+                np.savetxt(outf, out)
+        except click.Abort:
+            raise
+        except Exception as ex:
+            clk.print_except(ex, kwargs['debug'])
+    return 'logist-error', kwargs, ctx
+
+
+@main.command()
+@click.argument('coef', callback=clk.is_file)
+@click.option('-x', type=int, default=0,
+              help='column of training x vals')
+@click.option('-y', type=int, default=-1,
+              help='column of training y vals')
+@click.option('-rng', default=None, callback=clk.split_float,
+              help='if none, returns values from dataf (required if coef)')
+@click.option('--sort/--no-sort', default=False)
+@click.option('-outc', default=None,
+              help='if given, save coefs to file')
+@clk.shared_decs(clk.command_decs(hs.__version__))
+def softmax(ctx, coef, **kwargs):
+    """calculate softmax based on coef file (from )"""
+    
+    if kwargs['opts']:
+        kwargs['opts'] = False
+        clk.echo_args(coef, **kwargs)
+    else:
+        try:
+            coefs = np.loadtxt(coef)
+            if coefs.size != 6:
+                x = coefs[:,kwargs['x']]
+                y = coefs[:,kwargs['y']]
+                w = {0:1,1:len(x[y == 0])/len(y[y != 0]),2:len(y[y == 0])/len(y[y != 0])}
+                coefs = hs.train_logit(x, y, w)
+                if outc is not None:
+                    np.savetxt(out, coefs)
+            if kwargs['rng'] is None:
+                rng = x
+            else:
+                rng = np.array(kwargs['rng'])
+            if kwargs['sort']:
+                rng = np.sort(rng)
+            for r, li in zip(rng, hs.softmax_c(rng, coefs).T):
+                print(r, np.argmax(li)/(len(li)-1) , *li)
+        except click.Abort:
+            raise
+        except Exception as ex:
+            clk.print_except(ex, kwargs['debug'])
+    return 'softmax', kwargs, ctx
+
+
 # @main.command()
 # @click.argument('arg1')
 # @clk.shared_decs(clk.command_decs(hs.__version__))
